`timescale 1ns / 1ps
//----------------------------------------------------------------------//
//
//	Copyright (c) 2017 BeeBeans Technologies All rights reserved
//
//		System		: independent
//
//		Module		: ADC_CTL
//
//		Description	: SPI bus Control Core
//					  32-CLK Dual-SDO mode
//
//		file		: ADC_CTL.v
//
//		Note		:
//
//		history	:
//			170928	------		Created by H.Obo
//
//----------------------------------------------------------------------//


module ADC_CTL #(
		parameter	[ 7:0]	FREQUENCY	= 8'd200,		// CLK_IN Frequency  > 10MHz
		parameter	integer	DRIVE		= 4,		// Output Buffer Strength current
		parameter			IOSTANDARD	= "DEFAULT",	// I/O Standard
		parameter			SLEW		= "SLOW"		// Outputbufer Slew rate
	)(
		// SPI INTERCONNECT
		input	wire			CLK_IN,					// System Clock
		input	wire			RESET_IN,				// Reset
		input	wire			SPI_REQ_IN,				// SPI Request
		input	wire	[ 15:0]	SPI_CFG_REG_IN,			// SPI Configuration Register Write Data[15:0] default:0b1000_0010_0100_0000		

		output	wire			SPI_RAK_OUT,			// SPI Request Acknowledge
		output	reg		[ 11:0]	SPI_RDT_A_OUT,			// SPI Read Data[11:0]
		output	reg		[ 11:0]	SPI_RDT_B_OUT,			// SPI Read Data[11:0]
		output	reg				SPI_RVL_OUT,			// SPI Read Data Valid

		// SPI INTERFACE
		output	wire 			SPI_CLK_OUT,			//
		output	wire			SPI_CSn_OUT,			//	CHIP SELECT (active low)
		output	wire			SPI_MOSI_OUT,			//
		input	wire			SPI_MISO_A,				//
		input	wire			SPI_MISO_B				//
    );
    
    reg		SPI_CLK;
    reg		SPI_CSn;
    reg		SPI_MOSI;
    reg		[ 8:0]	CNT_1US;
    reg		[15:0]	SPI_CFG_REG;
    reg		SPI_CLK_d1;				//1 systemCLK delay than SPI_CLK, 
    reg		[ 7:0]	SPI_COUNTER;
    reg		[ 7:0]	SPI_COUNTER_D1;
    reg		SPI_CSn_REQ;
    reg		[15:0]	SPI_CFG_REG_SIFT;
    reg		[ 7:0]	CLK_DELAY;
    reg		[ 11:0]	SPI_RDT_A;
    reg		[ 11:0]	SPI_RDT_B;
    //////////////////////////////////////////////////
    // SPI CLK GEN
    //////////////////////////////////////////////////
	always@(posedge CLK_IN or posedge RESET_IN)begin
		if (RESET_IN) begin
			CNT_1US[8:0]	<= 8'd0;
			SPI_CLK			<= 1'b0;
			SPI_CLK_d1		<= 1'b0;
			CLK_DELAY[7:0]	<= 8'd0;
		end else begin
			CNT_1US[8:0]	<= CNT_1US[8]?		(FREQUENCY - 2)	:		(CNT_1US[8:0] - 9'd1)	;
			SPI_CLK			<= CNT_1US[8]?		~SPI_CLK		:		SPI_CLK					;	//WARNING 1 delay than CNT_1US[8]
			SPI_CLK_d1		<= SPI_CLK;
			CLK_DELAY[7:0]	<= CNT_1US[8]?		8'd0			:		CLK_DELAY[7:0]+8'd1		;
		end
	end
	
	//////////////////////////////////////////////////
	// SPI INTERCONNECT ACK
	//////////////////////////////////////////////////
	assign SPI_RAK_OUT = (SPI_CSn & SPI_REQ_IN & ~SPI_CSn_REQ);
	
	always@(posedge CLK_IN or posedge RESET_IN)begin
		if (RESET_IN) begin
			SPI_CFG_REG[15:0]	<= 16'd0;
			SPI_CSn_REQ			<= 1'd0;

		end else begin
			SPI_CFG_REG[15:0] 	<= (SPI_RAK_OUT)?	SPI_CFG_REG_IN[15:0]	: SPI_CFG_REG[15:0];
			SPI_CSn_REQ			<= (SPI_RAK_OUT)?	1'b1			: (SPI_CSn?	SPI_CSn_REQ:	1'b0);	//until SPI_CSn LOW(ACTIVE), SPI_CSn_REQ active HIGH.
		end
	end
	
	//////////////////////////////////////////////////
	// SPI INTERFACE
	//////////////////////////////////////////////////
	
	wire CSn_READY;		//CSn Acctive Low flag
	wire CSn_END;		//CSn END flag
	wire SPI_RDT_GET;	//SPI MISO TRIGGER
	wire SPI_CLK_TRGn;	//SPI CLK Negedge TRIGGER
	
	assign CSn_READY = (SPI_CSn & ~SPI_CLK & SPI_CSn_REQ & (CLK_DELAY[7:0] == 8'd30));
	assign CSn_END = (~SPI_CSn & SPI_CLK & ~SPI_CLK_d1 & (SPI_COUNTER[7:0] == 8'd0));
	assign SPI_RDT_GET = ((SPI_COUNTER[7:0] <= 8'd16)&(SPI_COUNTER[7:0] >= 8'd5)&(SPI_CLK_TRGn));
	assign SPI_CLK_TRGn = ~SPI_CLK & SPI_CLK_d1 & ~SPI_CSn;
	
	always@(posedge CLK_IN or posedge RESET_IN)begin
		if (RESET_IN) begin
			SPI_CSn				<= 1'b1;
			SPI_COUNTER[7:0]	<= 8'd32;
			SPI_COUNTER_D1[7:0]	<= 8'd32;
			SPI_MOSI			<= 1'b0;
			SPI_CFG_REG_SIFT[15:0]	<= 16'd0;
			SPI_RDT_A[11:0]		<=	12'd0;
			SPI_RDT_B[11:0]		<=	12'd0;
			SPI_RDT_A_OUT[11:0]		<=	12'd0;
			SPI_RDT_B_OUT[11:0]		<=	12'd0;
			SPI_RVL_OUT				<= 1'b0;
		end else begin
			// SPI_CSn will be LOW after SPI_CLK was LOW, Making a time delay.
			SPI_CSn					<= CSn_READY?		1'b0: (CSn_END?	1'b1	:	SPI_CSn);		//SPI_CLK negedge
			SPI_COUNTER_D1[7:0]		<= SPI_COUNTER[7:0];
			// SPI_COUNTER start from 32 until -1.Do countdown when clk is negedge.
			SPI_COUNTER[7:0]		<= SPI_CLK_TRGn?			SPI_COUNTER[7:0]-8'd1:	(SPI_CSn? 8'd32		:	SPI_COUNTER[7:0]);
			// When SPI_CSn is HIGH(NOT ACTIVE), assign SPI_CFG_REG. When SPI_CSn is LOW and CLK is negedge, SIFT this resister.
			SPI_CFG_REG_SIFT[15:0]	<= CSn_READY?	SPI_CFG_REG[15:0]:	(SPI_CLK_TRGn?		SPI_CFG_REG_SIFT[15:0]<<1 : SPI_CFG_REG_SIFT[15:0]	);	//negedge
			SPI_MOSI				<= (SPI_CLK & ~SPI_CLK_d1 & ~SPI_CSn)? SPI_CFG_REG_SIFT[15] : SPI_MOSI;//SPI_CLK posedge increment MOSI data
			// When SPI_COUNTER is 16,start MISO until 5. Get data when clk is negedge.
			SPI_RDT_A[11:0]		<= CSn_READY?	12'b0:(SPI_RDT_GET?	{SPI_RDT_A[10:0],SPI_MISO_A}:SPI_RDT_A[11:0]);
			SPI_RDT_B[11:0]		<= CSn_READY?	12'b0:(SPI_RDT_GET?	{SPI_RDT_B[10:0],SPI_MISO_B}:SPI_RDT_B[11:0]);
			SPI_RDT_A_OUT[11:0] <= (SPI_COUNTER[7:0] == 8'd0)?	SPI_RDT_A[11:0]:	SPI_RDT_A_OUT[11:0];
			SPI_RDT_B_OUT[11:0] <= (SPI_COUNTER[7:0] == 8'd0)?	SPI_RDT_B[11:0]:	SPI_RDT_B_OUT[11:0];
			// When turn to SPI_CSn(NON-ACTIVE), Valid READ DATA.
			//SPI_RVL_OUT				<= CSn_END?	1'b1:	1'b0;
			SPI_RVL_OUT			<= SPI_COUNTER[7:0] == 8'd0	&	(SPI_COUNTER[7:0] != SPI_COUNTER_D1[7:0])	&	~SPI_RVL_OUT;
		end
	end
	
	OBUF	#(.DRIVE(DRIVE), .IOSTANDARD(IOSTANDARD), .SLEW(SLEW))	SPI_CLKL_OB	(				.O (SPI_CLK_OUT),	.I(SPI_CLK));
	OBUF	#(.DRIVE(DRIVE), .IOSTANDARD(IOSTANDARD), .SLEW(SLEW))	SPI_CSn_OB	(				.O (SPI_CSn_OUT),	.I(SPI_CSn));
	OBUF	#(.DRIVE(DRIVE), .IOSTANDARD(IOSTANDARD), .SLEW(SLEW))	SPI_MOSI_OB	(				.O (SPI_MOSI_OUT),	.I(SPI_MOSI));
	
endmodule
