//----------------------------------------------------------------------//
//
//	Copyright (c) 2020 BeeBeans Technologies All rights reserved
//
//		System		: independent
//
//		Module		: DAC_CTL
//
//		Description	: DAC control
//
//		file		: DAC_CTL.v
//
//		Note		: 
//
//		history	:
//			200312	------		Created by A. Tsuchiya
//
//----------------------------------------------------------------------//
`default_nettype none

module 
	DAC_CTL #(
		parameter	[ 7:0]	FREQUENCY	= 8'd200		// CLK_IN Frequency  > 10MHz
	)(
		input	wire			CLK_IN,				// System Clock
		input	wire			RESET_IN,			// Reset
		output	wire			IIC_SCL_OUT,		//I2C interface SCL
		input	wire			IIC_SDA_IO,			//I2C interface SDA
		
		input	wire	[11:0]	DAC_WDT,			//Write Data DAC
		input	wire	[1:0]	DAC_CH,				//DAC CH
		input	wire			DAC_REQ,			//DAC Request
		
		output	reg			DAC_RAK_OUT				//DAC Request Acknowledge
		
	);
	
	wire			IIC_BSY;							// IIC BUSY
	wire			IIC_RAK;							// IIC Request Acknowledge
	reg				IIC_REQ;							// IIC Request
	reg		[7:0]	IIC_WDT;							// IIC write Data[7:0]
	reg		[3:0]	IIC_DATA_CNT;						// IIC Data Count
	wire			IIC_WDA;							// IIC Wite Data Acknowledge
	wire			IIC_WAE;							// IIC Wite Last Data Acknowledge(same as IIC_WDA timing)
	
	reg		[4:0]	DAC_WMODE_MULTI 	= 	5'b01000;	// Multi-Write Command: Write Multiple DAC Input Registers
	reg		[1:0]	DAC_W_ST_CH			=	2'b00;		// Channel Select
	reg		[11:0]	DAC_WDT_UPD;						// DAC Wite Data Update
	reg		[1:0]	DAC_CH_UPD;							// DAC channel Update
	
	IIC_CORE 	#(
		.FREQUENCY		(FREQUENCY),		// CLK_IN Frequency  > 10MHz
		.DRIVE			(4),				// Output Buffer Strength
		.IOSTANDARD		("DEFAULT"),			// I/O Standard
		.SLEW			("SLOW")					// Outputbufer Slew rate
	)
	IIC_CORE		(
		.CLK_IN			(CLK_IN),				// System Clock
		.RESET_IN		(RESET_IN),				// Reset
		.IIC_REQ_IN		(IIC_REQ),				// IIC Request
		.IIC_AST_IN		(1'b0),					// IIC Address Set Request
		.IIC_NUM_IN		(8'd1),					// IIC Number of Access[7:0]	0x00:1Byte , 0xff:256Byte
		.IIC_DAD_IN		(7'b1100_000),			// IIC Device Address[6:0]
		.IIC_RNW_IN		(1'b0),					// IIC Read(1) / Write(0)
		.IIC_WDT_IN		(IIC_WDT[7:0]),			// IIC Write Data[7:0]
		.IIC_RAK_OUT	(IIC_RAK),				// IIC Request Acknowledge
		.IIC_WDA_OUT	(IIC_WDA),				// IIC Wite Data Acknowledge(Next Data Request)
		.IIC_WAE_OUT	(IIC_WAE),				// IIC Wite Last Data Acknowledge(same as IIC_WDA timing)
		.IIC_BSY_OUT	(IIC_BSY),				// IIC Busy
		.IIC_RDT_OUT	(),						// IIC Read Data[7:0]
		.IIC_RVL_OUT	(),						// IIC Read Data Valid
		.IIC_EOR_OUT	(),						// IIC End of Read Data(same as IIC_RVL timing)
		// Device Interface
		.IIC_SCL_OUT	(IIC_SCL_OUT),			// IIC Clock
		.IIC_SDA_IO		(IIC_SDA_IO)			// IIC Data
	);
	
	always@ (posedge CLK_IN) begin
		if(RESET_IN) begin
			IIC_REQ					<= 1'b0;
			IIC_DATA_CNT[3:0]		<= 4'b0;
			IIC_WDT[7:0]			<= 8'b0;
			DAC_RAK_OUT             <= 1'b0;
			DAC_WDT_UPD              <=  12'd0;
		end else begin
		//IIC Request
		IIC_REQ			<= (
			(~IIC_REQ & (DAC_REQ))|
			(IIC_REQ & ~IIC_RAK)
		);
		
		// Data Count
		IIC_DATA_CNT[3:0]	<=	(
			(IIC_WAE ?	4'b0000:	(IIC_DATA_CNT[3:0] + IIC_WDA))
			);
		
		//Data and Channel Update (if DAC Request Acknowledge == HIGH)
		DAC_WDT_UPD[11:0]	<=	(DAC_REQ	&	~IIC_BSY)	?	 DAC_WDT[11:0]	:	 DAC_WDT_UPD[11:0];
		DAC_CH_UPD[1:0]	<=	(DAC_REQ	&	~IIC_BSY)	?	 DAC_CH[1:0]	:	 DAC_CH_UPD[1:0];
		
		//IIC write Data
		IIC_WDT[7:0]	<=	(
			(IIC_DATA_CNT[3:0] == 4'b0000 ?	{DAC_WMODE_MULTI[4:0],DAC_CH_UPD[1:0],1'b0}		:		8'b0)|	//IIC Mode Select
			(IIC_DATA_CNT[3:0] == 4'b0001 ?	{4'b1000,DAC_WDT_UPD[11:8]}			:		8'b0)|			//Write DAC Data[11:8]
			(IIC_DATA_CNT[3:0] == 4'b0010 ?	{DAC_WDT_UPD[7:0]}			:		8'b0)					//Write DAC Data[7:0]
		);
		
		DAC_RAK_OUT	<=	DAC_REQ	&	~IIC_BSY &   ~DAC_RAK_OUT;	//DAC Reqest Acknowledge
		end
	end
endmodule
`default_nettype wire
